diff -urN lib.orig/Mail/SpamAssassin/Message/Node.pm lib/Mail/SpamAssassin/Message/Node.pm
--- lib.orig/Mail/SpamAssassin/Message/Node.pm	Fri Oct 22 18:21:03 2004
+++ lib/Mail/SpamAssassin/Message/Node.pm	Mon Apr  4 15:45:29 2005
@@ -57,6 +57,8 @@
   my $self = {
     headers		=> {},
     raw_headers		=> {},
+    headers_case	=> {},
+    raw_headers_case	=> {},
     body_parts		=> [],
     header_order	=> []
   };
@@ -173,6 +175,14 @@
     push @{ $self->{'headers'}->{$key} },     _decode_header($raw_value);
     push @{ $self->{'raw_headers'}->{$key} }, $raw_value;
 
+    if ( !exists $self->{'headers_case'}->{$rawkey} ) {
+      $self->{'headers_case'}->{$rawkey} = [];
+      $self->{'raw_headers_case'}->{$rawkey} = [];
+    }
+
+    push @{ $self->{'headers_case'}->{$rawkey} },     _decode_header($raw_value);
+    push @{ $self->{'raw_headers_case'}->{$rawkey} }, $raw_value;
+
     return $self->{'headers'}->{$key}->[-1];
   }
 
@@ -186,6 +196,28 @@
   }
 }
 
+sub header_case {
+  my $self   = shift;
+  my $rawkey = shift;
+
+  return unless ( defined $rawkey );
+
+  my $key    = $rawkey;
+
+  # Trim whitespace off of the header keys
+  $key       =~ s/^\s+//;
+  $key       =~ s/\s+$//;
+
+  if (wantarray) {
+    return unless exists $self->{'headers_case'}->{$key};
+    return @{ $self->{'headers_case'}->{$key} };
+  }
+  else {
+    return '' unless exists $self->{'headers_case'}->{$key};
+    return $self->{'headers_case'}->{$key}->[-1];
+  }
+}
+
 =item raw_header()
 
 Retrieves the raw version of headers from a specific MIME part.  The only
@@ -221,6 +253,24 @@
   }
 }
 
+sub raw_header_case {
+  my $self = shift;
+  my $key  = shift;
+
+  # Trim whitespace off of the header keys
+  $key       =~ s/^\s+//;
+  $key       =~ s/\s+$//;
+
+  if (wantarray) {
+    return unless exists $self->{'raw_headers_case'}->{$key};
+    return @{ $self->{'raw_headers_case'}->{$key} };
+  }
+  else {
+    return '' unless exists $self->{'raw_headers_case'}->{$key};
+    return $self->{'raw_headers_case'}->{$key}->[-1];
+  }
+}
+
 =item add_body_part()
 
 Adds a Node child object to the current node object.
@@ -485,6 +535,8 @@
   foreach ( grep(/^${hdr}$/i, keys %{$self->{'headers'}}) ) {
     delete $self->{'headers'}->{$_};
     delete $self->{'raw_headers'}->{$_};
+    delete $self->{'headers_case'}->{$_};
+    delete $self->{'raw_headers_case'}->{$_};
   }
   
   my @neworder = grep(!/^${hdr}$/i, @{$self->{'header_order'}});
@@ -575,6 +627,35 @@
   }
 }
 
+sub get_header_case {
+  my ($self, $hdr, $raw) = @_;
+  $raw ||= 0;
+
+  # And now pick up all the entries into a list
+  # This is assumed to include a newline at the end ...
+  # This is also assumed to have removed continuation bits ...
+
+  # Deal with the possibility that header() or raw_header() returns undef
+  my @hdrs;
+  if ( $raw ) {
+    if (@hdrs = $self->raw_header_case($hdr)) {
+      @hdrs = map { s/\r?\n\s+/ /g; $_; } @hdrs;
+    }
+  }
+  else {
+    if (@hdrs = $self->header_case($hdr)) {
+      @hdrs = map { "$_\n" } @hdrs;
+    }
+  }
+
+  if (wantarray) {
+    return @hdrs;
+  }
+  else {
+     return @hdrs ? $hdrs[-1] : undef;
+  }
+}
+
 =item get_all_headers()
 
 Retrieve all headers.  Each header will have a newline at the end and
@@ -644,6 +725,8 @@
   # Clean up ourself
   undef $self->{'headers'};
   undef $self->{'raw_headers'};
+  undef $self->{'headers_case'};
+  undef $self->{'raw_headers_case'};
   undef $self->{'header_order'};
   undef $self->{'raw'};
   undef $self->{'decoded'};
diff -urN lib.orig/Mail/SpamAssassin/PerMsgStatus.pm lib/Mail/SpamAssassin/PerMsgStatus.pm
--- lib.orig/Mail/SpamAssassin/PerMsgStatus.pm	Thu Dec 16 05:10:03 2004
+++ lib/Mail/SpamAssassin/PerMsgStatus.pm	Mon Apr  4 15:45:29 2005
@@ -1348,10 +1348,25 @@
     $_ = $self->{hdr_cache}->{$request};
   }
   else {
+    if ($request =~ /\|/) {
+	foreach my $subrequest (split(/\|/, $request)) {
+	    my $getcase = ($subrequest =~ s/:case//);
+	    my $getraw = ($subrequest eq 'ALL' || $subrequest =~ s/:raw$//);
+	    if ($getcase) {
+		$_ .= '|'.join ("\n", $self->{msg}->get_header_case($subrequest, $getraw));
+	    } else {
+		$_ .= '|'.join ("\n", $self->{msg}->get_header($subrequest, $getraw));
+	    }
+	}
+	s/^\|//;
+        return($_);
+    }
+
     my $hdrname = $request;
     my $getaddr = ($hdrname =~ s/:addr$//);
     my $getname = ($hdrname =~ s/:name$//);
     my $getraw = ($hdrname eq 'ALL' || $hdrname =~ s/:raw$//);
+    my $getcase = ($hdrname =~ s/:case//);
 
     if ($hdrname eq 'ALL') {
       $_ = $self->{msg}->get_all_headers($getraw);
@@ -1392,7 +1407,12 @@
     }
     # a conventional header
     else {
-      my @hdrs = $self->{msg}->get_header ($hdrname, $getraw);
+      my @hdrs;
+      if ($getcase) {
+        @hdrs = $self->{msg}->get_header_case ($hdrname, $getraw);
+      } else {
+        @hdrs = $self->{msg}->get_header ($hdrname, $getraw);
+      }
       if ($#hdrs >= 0) {
         $_ = join ('', @hdrs);
       }
